package com.bweb.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.bweb.config.SysConst;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.AESEncodeUtil;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.CommonSysConst;
import com.hai.entity.*;
import com.hai.model.DiscountUseConditionModel;
import com.hai.model.ResponseData;
import com.hai.model.UserInfoModel;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/4/4 00:19
 */
@Controller
@RequestMapping(value = "/discountAgentBatch")
@Api(value = "优惠券和代理商接口")
public class HighDiscountAgentBatchController {

    private static Logger log = LoggerFactory.getLogger(HighDiscountAgentBatchController.class);
    @Resource
    private HighDiscountAgentBatchService highDiscountAgentBatchService;

    @RequestMapping(value="/replenish",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "批次补充")
    public ResponseData replenish(@RequestBody JSONObject jsonObject) {
        try {

            String batchIdentifier = jsonObject.getString("batchIdentifier");
            Long mailTime = jsonObject.getLong("mailTime");
            Integer batchNum = jsonObject.getInteger("batchNum");

            if (StringUtils.isBlank(batchIdentifier)) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }
            // 查询数据
            HighDiscountAgentBatch batch = highDiscountAgentBatchService.getBatchByIdentifier(batchIdentifier);
            if (batch == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "批次不存在");
            }
            if (mailTime != null) {
                batch.setMailTime(new Date(mailTime));
            }
            if (batchNum != null) {
                batch.setBatchNum(batch.getBatchNum().intValue() + batchNum);
            }
            highDiscountAgentBatchService.editData(batch);
            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighDiscountAgentBatchController -> replenish() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getBatchDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询批次详情")
    public ResponseData getBatchDetail(@RequestParam(name = "batchIdentifier", required = true) String batchIdentifier) {
        try {

            return ResponseMsgUtil.success(highDiscountAgentBatchService.getBatchByIdentifier(batchIdentifier));

        } catch (Exception e) {
            log.error("HighDiscountController -> getBatchDetail() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
